/*
   COM- : 
9600 /, 1 -, 8  ,   

 :
1)    COM- 
2)  +5
3)   20 ( RS232),   5A 20 7A, 
4)  +12
5)   21 ( Fuse-,  )
6)   22   Fuse-  Lock- 
7)    +12
8)    +5
9)    COM- 
10)  ATmega8      
  
==================*/


/*Fuse-  ATmega8535:
  1101 1001		high
  1110 1111 	low*/

.include	"m8535def.inc"
.equ	RDY	=PB0
.equ	S12V	=PB1
.equ 	S5V	=PB2

.equ	OE	=PA0
.equ	WR	=PA1
.equ	BS1	=PA2
.equ	XTAL	=PA3
.equ	XA0	=PA4
.equ	XA1	=PA5
.equ	PAGEL	=PA6
.equ	BS2	=PA7

.equ	cmdErase	=0b10000000
.equ	cmdFuse	=0b01000000
.equ	cmdReadFuse	=0b00000100

/*Fuse- ATmega8:
 (,   
  )*/
.equ	HiFuse	=0b11011001	
.equ	LoFuse	=0b11100001

.def	tm1	=r16
.def	tm2	=r17
.def	tmp	=r18

.def	dl1	=r20


.def	datrs	=r24
.def	cmdrs	=r25

;.equ	RXD	=PD0
.equ	TXD	=PD1

.equ	FrstRx	=$5a	;   

.equ	cmRS		=$20	;  RS-232
.equ	cmProgFuse	=$21	;  Fuse-
.equ	cmRdFuse	=$22	;  Fuse-  Lock-
.equ	cmdEraseProgFuse	=$23	;  Fuse-
				;   Lock-  
.equ	tick0	=30
.equ	pre0	=5 	;4ms

.CSEG
.org 0
	rjmp	RESET
.org	OVF0addr
	rjmp	T0_Ovf
.org URXCaddr
	rjmp	U_RXC 	; USART1 Receive Complete Interrupt

.MACRO	UInit
	ldi	tm1,(1<<RXCIE)|(1<<RXEN)
	out	UCSRB,tm1
	ldi	tm1,(1<<URSEL)|(3<<UPM0)|(3<<UCSZ0)
	out	UCSRC,tm1
	ldi	ZL,low(SRAM_START)
	ldi	ZH,high(SRAM_START)
.ENDMACRO

;=== :===
U_RXC:in	datrs,UDR
	st	Z+,datrs
	ldi	tm2,tick0
	out	TCNT0,tm2
	ldi	tm2,pre0
	out	TCCR0,tm2
	reti

T0_Ovf:
	clr	tm2
	out	TCCR0,tm2
	ldi	ZL,low(SRAM_START)
	ldi	ZH,high(SRAM_START)
	ld	cmdrs,Z+
	reti
;===

;===:===
;=== :===
TxCh:	ldi	tm1,1<<TXEN
	out	UCSRB,tm1
	ldi	tm1,1<<TXC
	out	UCSRA,tm1
mTx:	sbis	UCSRA,UDRE
	rjmp	mTx
	out	UDR,datrs
	ret
; TxCh	
	
;===   ===
TxEnd:	sbis	UCSRA,TXC	;  
	rjmp	TxEnd
	sbi	PORTD,TXD
	UInit	47	
	ret

;=== RS-232, : 5A 20 7A:===
xTstRS:
	ldi	datrs,FrstRx
	rcall	TxCh
	mov	datrs,cmdrs
	rcall	TxCh
	ldi	tm1,FrstRx
	add	datrs,tm1
	rcall	TxCh
	rcall	TxEnd
	ret

;=== :===
ExeCmd:
	cbi	UCSRB,RXCIE
	cpi	cmdrs,cmRS
	brne	E1
	rcall	xTstRS
E1:	cpi	cmdrs,cmProgFuse
	brne	E2
	rcall	xProgFuse
E2:	cpi	cmdrs,cmRdFuse
	brne	E3
	rcall	xRdFuse
E3:	cpi	cmdrs,cmdEraseProgFuse
	brne	E4
	rcall	xProgFuse
E4:	clr	cmdrs
	UInit
	ret

/*     
   Fuse- RESET=0  CKSEL[3:0]<>0b0000*/
SetEmrgPRGmode:
	ldi	tmp,(0<<PAGEL)|(0<<XA1)|(0<<XA0)|(0<<BS1)| (1<<OE)|(1<<WR)
	out	PORTA,tmp
	rcall	Delay

	ldi	tmp,(1<<RDY)|(1<<S5V)|(1<<S12V)
	out	PORTB,tmp
	rcall	Delay
	ret

/*     
   Fuse- RESET<>0  CKSEL[3:0]=0b0000*/
SetCmnPRGmode:
	sbi	PORTB,S5V
	rcall	Delay		
	cbi	PORTB,S12V
	rcall	PlsXTal
	rcall	PlsXTal
	rcall	PlsXTal
	rcall	PlsXTal
	rcall	PlsXTal
	rcall	PlsXTal
	rcall	PlsXTal
	rcall	PlsXTal
	ldi	tmp,(0<<PAGEL)|(0<<XA1)|(0<<XA0)|(0<<BS1)|(1<<WR)|(1<<OE)
	out	PORTA,tmp
	rcall	Delay

	sbi	PORTB,S12V
	rcall	Delay
	ret

;===   :===
SetNeutralMode:
	cbi	PORTB,S12V
	rcall	Delay
	cbi	PORTB,S5V
	ret

;===   Lock-:===
xErase:
	sbi	PORTA,XA1	;XA1:XA0=10,BS1=0
	cbi	PORTA,XA0
	cbi	PORTA,BS1
	ldi	tmp,cmdErase
	out	PORTC,tmp	
	rcall	PlsXTAL
	rcall	PlsWr
xEr1:	wdr
	sbis	PINB,RDY
	rjmp	xEr1
	ret

;=== Fuse-:===
xProgFuse:
	rcall	SetEmrgPrgMode

	cpi	cmdrs,cmdEraseProgFuse
	brne	xProg1	;   cmdEraseProgFuse,
	rcall	xErase	;   Lock-

;===  Fuse-:===
;  0100 0000:
xProg1:
	sbi	PORTA,XA1	;XA1:XA0=10,BS1=0
	cbi	PORTA,XA0
	cbi	PORTA,BS1
	cbi	PORTA,BS2
	ldi	tmp,cmdFuse
	out	PORTC,tmp	
	rcall	PlsXTAL
	cbi	PORTA,XA1
	
;    Fuse-		
	cbi	PORTA,XA1
	sbi	PORTA,XA0
	ldi	tmp,LoFuse 
	out	PORTC,tmp
	rcall	PlsXTAL

; :
	cbi	PORTA,BS2
	cbi	PORTA,BS1
	rcall	PlsWr
WtRdy:wdr
	sbis	PINB,RDY
	rjmp	WtRdy
	rcall	Delay

;===  Fuse-:===
;  0100 0000:

	sbi	PORTA,XA1	;XA1:XA0=10,BS1=0
	cbi	PORTA,XA0
	cbi	PORTA,BS1
	cbi	PORTA,BS2
	ldi	tmp,cmdFuse
	out	PORTC,tmp	
	rcall	PlsXTAL
	cbi	PORTA,XA1
	
;    Fuse-		
	cbi	PORTA,XA1
	sbi	PORTA,XA0
	ldi	tmp,HiFuse
	out	PORTC,tmp
	;out	PORTC,dataHiFuse
	rcall	PlsXTAL

; :
	cbi	PORTA,BS2
	sbi	PORTA,BS1
	rcall	PlsWr
WRdy:	wdr
	sbis	PINB,RDY
	rjmp	WRdy
	rcall	Delay

	cbi	PORTA,BS1

;   
	rcall	SetNeutralMode
; :
	mov	datrs,cmdrs
	rcall	TxCh
	rcall	TxEnd
	ret

;=== Fuse-  Lock-
xRdFuse:
	rcall	SetCmnPRGmode

; ReadFuse:
	sbi	PORTA,WR
	sbi	PORTA,OE

	sbi	PORTA,XA1	;XA1:XA0=10,BS1=0
	cbi	PORTA,XA0
	cbi	PORTA,BS1
	cbi	PORTA,BS2
	ldi	tmp,cmdReadFuse
	out	PORTC,tmp	
	rcall	PlsXTAL
	cbi	PORTA,XA1

	cbi	PORTA,OE
	cbi	PORTA,BS2
	cbi	PORTA,BS1

	clr	tmp	
	out	DDRC,tmp
	ser	tmp
	out	PORTC,tmp

;:
	rcall	Delay
	in	datrs,PINC	;LowFuse
	rcall	TxCh

	sbi	PORTA,BS2
	sbi	PORTA,BS1
	rcall	Delay
	in	datrs,PINC	;HighFuse
	rcall	TxCh

	cbi	PORTA,BS2
	sbi	PORTA,BS1
	rcall	Delay
	in	datrs,PINC	;LockBits
	rcall	TxCh

	rcall	TxEnd
	sbi	PORTA,OE
	ser	tmp
	out	DDRC,tmp
	rcall	SetNeutralMode
	ret
	
;=== 140us:=== 
Delay:
	ldi	dl1,255
Del1:	wdr
	dec	dl1
	brne	Del1
	ret

;===   XTAL1:===
PlsXTAL:
	rcall	Delay
	sbi	PORTA,XTAL
	rcall	Delay
	cbi	PORTA,XTAL
	rcall	Delay
	ret

;   Wr^===
PlsWr:
	cbi	PORTA,Wr
	rcall	Delay
	sbi	PORTA,Wr
	rcall	Delay
	ret


;=== :===
Reset:ldi	tmp,(1<<WDE)|(1<<WDCE)
	out	WDTCR,tmp
	ldi	tmp,(1<<WDE)|(1<<WDP2)|(1<<WDP1)|(1<<WDP0)
	out	WDTCR,tmp
	wdr

	ldi	tmp,HIGH(RAMEND)	; 
	out	SPH,tmp		
	ldi	tmp,LOW(RAMEND)	
	out	SPL,tmp		

	ldi	tmp,$ff
	out	DDRA,tmp
	out	DDRC,tmp
	out	DDRD,tmp
	out	PORTD,tmp
	clr	tmp
	out	PORTC,tmp
	ldi	tmp,(1<<XA1)|(1<<XA0)
	out	PORTA,tmp


	ldi	tmp,~(1<<RDY)
	out	DDRB,tmp
	ldi	tmp,1<<RDY
	out	PORTB,tmp

	ldi	tm1,high(47)		;9600 /
	out	UBRRH,tm1
	ldi	tm1,low(47)
	out	UBRRL,tm1
	UInit	

	clr	cmdrs
	ldi	tm1,1<<TOIE0
	out	TIMSK,tm1
	sei

;===   :===
cycle:wdr
	tst	cmdrs
	breq	cycle
	rcall	ExeCmd
	rjmp	cycle
	
